@@ -34,7 +34,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcp/tcp.cc,v 1.163 2005/06/21 01:48:24 sfloyd Exp $ (LBL)";
+    "@(#) $Header: /cvsroot/nsnam/ns-2/tcp/tcp.cc,v 1.173 2007/01/12 18:08:52 sallyfloyd Exp $ (LBL)";
 #endif
 
 #include <stdlib.h>
@@ -73,7 +73,8 @@ TcpAgent::TcpAgent()
 	  rtx_timer_(this), delsnd_timer_(this), burstsnd_timer_(this), 
 	  dupacks_(0), curseq_(0), highest_ack_(0), cwnd_(0), ssthresh_(0), 
 	  maxseq_(0), count_(0), rtt_active_(0), rtt_seq_(-1), rtt_ts_(0.0), 
-	  lastreset_(0.0), closed_(0), first_decrease_(1), fcnt_(0), 
+	  lastreset_(0.0), closed_(0), use_rtt_(0),
+	  first_decrease_(1), fcnt_(0), 
 	  nrexmit_(0), restart_bugfix_(1), cong_action_(0), 
 	  ecn_burst_(0), ecn_backoff_(0), ect_(0), 
 	  qs_requested_(0), qs_approved_(0),
@@ -81,8 +82,8 @@ TcpAgent::TcpAgent()
 	
 {
 #ifdef TCP_DELAY_BIND_ALL
+        // defined since Dec 1999.
 #else /* ! TCP_DELAY_BIND_ALL */
-	// not delay-bound because delay-bound tracevars aren't yet supported
 	bind("t_seqno_", &t_seqno_);
 	bind("rtt_", &t_rtt_);
 	bind("srtt_", &t_srtt_);
@@ -126,6 +127,7 @@ TcpAgent::delay_bind_init_all()
         delay_bind_init_one("ecn_");
         delay_bind_init_one("SetCWRonRetransmit_");
         delay_bind_init_one("old_ecn_");
+        delay_bind_init_one("bugfix_ss_");
         delay_bind_init_one("eln_");
         delay_bind_init_one("eln_rxmit_thresh_");
         delay_bind_init_one("packetSize_");
@@ -184,6 +186,7 @@ TcpAgent::delay_bind_init_all()
 	delay_bind_init_one("qs_request_mode_");
 	delay_bind_init_one("qs_thresh_");
 	delay_bind_init_one("qs_rtt_");
+	delay_bind_init_one("print_request_");
 
 	delay_bind_init_one("frto_enabled_");
 	delay_bind_init_one("sfrto_enabled_");
@@ -234,6 +237,7 @@ TcpAgent::delay_bind_dispatch(const char *varName, const char *localName, TclObj
         if (delay_bind_bool(varName, localName, "ecn_", &ecn_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "SetCWRonRetransmit_", &SetCWRonRetransmit_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "old_ecn_", &old_ecn_ , tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "bugfix_ss_", &bugfix_ss_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "eln_", &eln_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "eln_rxmit_thresh_", &eln_rxmit_thresh_ , tracer)) return TCL_OK;
         if (delay_bind(varName, localName, "packetSize_", &size_ , tracer)) return TCL_OK;
@@ -290,12 +294,14 @@ TcpAgent::delay_bind_dispatch(const char *varName, const char *localName, TclObj
         if (delay_bind(varName, localName, "rate_request_", &rate_request_ , tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "qs_enabled_", &qs_enabled_ , tracer)) return TCL_OK;
 	if (delay_bind_bool(varName, localName, "tcp_qs_recovery_", &tcp_qs_recovery_, tracer)) return TCL_OK;
-
+	if (delay_bind(varName, localName, "qs_request_mode_", &qs_request_mode_, tracer)) return TCL_OK;
+	if (delay_bind(varName, localName, "qs_thresh_", &qs_thresh_, tracer)) return TCL_OK;
+	if (delay_bind(varName, localName, "qs_rtt_", &qs_rtt_, tracer)) return TCL_OK;
+	if (delay_bind_bool(varName, localName, "print_request_", &print_request_, tracer)) return TCL_OK;
 	if (delay_bind_bool(varName, localName, "frto_enabled_", &frto_enabled_, tracer)) return TCL_OK;
 	if (delay_bind_bool(varName, localName, "sfrto_enabled_", &sfrto_enabled_, tracer)) return TCL_OK;
 	if (delay_bind_bool(varName, localName, "spurious_response_", &spurious_response_, tracer)) return TCL_OK;
 
-
 #ifdef TCP_DELAY_BIND_ALL
 	// not if (delay-bound delay-bound tracevars aren't yet supported
         if (delay_bind(varName, localName, "t_seqno_", &t_seqno_ , tracer)) return TCL_OK;
@@ -452,6 +458,7 @@ TcpAgent::reset()
 	maxseq_ = -1;
 	last_ack_ = -1;
 	highest_ack_ = -1;
+	//highest_ack_ = 1;
 	ssthresh_ = int(wnd_);
 	if (max_ssthresh_ > 0 && max_ssthresh_ < ssthresh_) 
 		ssthresh_ = max_ssthresh_;
@@ -524,11 +531,12 @@ double TcpAgent::rtt_timeout()
 	if (rfc2988_) {
 	// Correction from Tom Kelly to be RFC2988-compliant, by
 	// clamping minrto_ before applying t_backoff_.
-		if (t_rtxcur_ < minrto_)
+		if (t_rtxcur_ < minrto_ && !use_rtt_)
 			timeout = minrto_ * t_backoff_;
 		else
 			timeout = t_rtxcur_ * t_backoff_;
 	} else {
+		// only of interest for backwards compatibility
 		timeout = t_rtxcur_ * t_backoff_;
 		if (timeout < minrto_)
 			timeout = minrto_;
@@ -542,9 +550,12 @@ double TcpAgent::rtt_timeout()
 			fprintf(stderr, "TcpAgent: negative RTO!  (%f)\n",
 				timeout);
 			exit(1);
-		}
-		timeout = 2.0 * tcp_tick_;
+		} else if (use_rtt_ && timeout < tcp_tick_)
+			timeout = tcp_tick_;
+		else
+			timeout = 2.0 * tcp_tick_;
 	}
+	use_rtt_ = 0;
 	return (timeout);
 }
 
@@ -699,21 +710,28 @@ void TcpAgent::output(int seqno, int reason)
 			// dataout is kilobytes queued for sending
 			int dataout = (curseq_ - maxseq_ - 1) * (size_ + headersize()) / 1024;
 			int qs_rr = rate_request_;
-			if (qs_request_mode_ == 1) {
+			if (qs_request_mode_ == 1 && qs_rtt_ > 0) {
 				// PS: Avoid making unnecessary QS requests
 				// use a rough estimation of RTT in qs_rtt_
 				// to calculate the desired rate from dataout.
+				// printf("dataout %d qs_rr %d qs_rtt_ %d\n",
+				//	dataout, qs_rr, qs_rtt_);
 				if (dataout * 1000 / qs_rtt_ < qs_rr) {
 					qs_rr = dataout * 1000 / qs_rtt_;
 				}
+				// printf("request %d\n", qs_rr);
 				// qs_thresh_ is minimum number of unsent
 				// segments needed to activate QS request
+				// printf("curseq_ %d maxseq_ %d qs_thresh_ %d\n",
+				//	 int(curseq_), int(maxseq_), qs_thresh_);
 				if ((curseq_ - maxseq_ - 1) < qs_thresh_) {
 					qs_rr = 0;
 				}
-			}
+			} 
 
 		    	if (qs_rr > 0) {
+				if (print_request_) 
+					printf("QS request (before encoding): %d KBps\n", qs_rr);
 				// QuickStart code from Srikanth Sundarrajan.
 				qsh->flag() = QS_REQUEST;
 				qsh->ttl() = Random::integer(256);
@@ -985,15 +1003,24 @@ double TcpAgent::linear(double x, double x_1, double y_1, double x_2, double y_2
 
 /*
  * Limited Slow-Start for large congestion windows.
- * This is only used when max_ssthresh_ is non-zero.
+ * This should only be called when max_ssthresh_ is non-zero.
  */
-double TcpAgent::limited_slow_start(double cwnd, double max_ssthresh, double increment)
+double TcpAgent::limited_slow_start(double cwnd, int max_ssthresh, double increment)
 {
-	int round = int(cwnd / (double(max_ssthresh)/2.0));
-	double increment1 = 1.0/(double(round)); 
-	if (increment < increment1)
-		increment = increment1;
-	return increment;
+        if (max_ssthresh <= 0) {
+	  	return increment;
+	} else {
+                double increment1 = 0.0;
+		int round = int(cwnd / (double(max_ssthresh)/2.0));
+		if (round > 0) {
+		  	increment1 = 1.0/double(round); 
+		} 
+		if (increment < increment1) {
+		  	return increment1;
+		} else {
+		  	return increment;
+		}
+	}
 }
 
 /*
@@ -1547,7 +1574,11 @@ void
 TcpAgent::dupack_action()
 {
 	int recovered = (highest_ack_ > recover_);
-	if (recovered || (!bug_fix_ && !ecn_)) {
+	if (recovered || (!bug_fix_ && !ecn_) || 
+		(bugfix_ss_ && highest_ack_ == 0)) {
+		// (highest_ack_ == 0) added to allow Fast Retransmit
+		//  when the first data packet is dropped.
+		//  Bug report from Mark Allman.
 		goto tahoe_action;
 	}
 
@@ -1610,12 +1641,17 @@ void TcpAgent::processQuickStart(Packet *pkt)
 	qs_approved_ = 0;
 	if (qsh->flag() == QS_RESPONSE && qsh->ttl() == ttl_diff_ && 
             qsh->rate() > 0) {
-                app_rate = (int) (hdr_qs::rate_to_Bps(qsh->rate()) *
-                      (now - tcph->ts_echo()) / (size_ + headersize()));
-#ifdef QS_DEBUG
-		printf("Quick Start approved, rate %d, window %d\n", 
-				     qsh->rate(), app_rate);
-#endif
+                app_rate = (int) ((hdr_qs::rate_to_Bps(qsh->rate()) *
+                      (now - tcph->ts_echo())) / (size_ + headersize()));
+		if (print_request_) {
+		  double num1 = hdr_qs::rate_to_Bps(qsh->rate());
+		  double time = now - tcph->ts_echo();
+		  int size = size_ + headersize();
+		  printf("Quick Start request, rate: %g Bps, encoded rate: %d\n", 
+		     num1, qsh->rate());
+		  printf("Quick Start request, window %d rtt: %4.2f pktsize: %d\n",
+		     app_rate, time, size);
+		}
                 if (app_rate > initial_window()) {
 			qs_cwnd_ = app_rate;
                         qs_approved_ = 1;
@@ -1831,6 +1867,14 @@ void TcpAgent::timeout(int tno)
 				 * initial windows. 
 				 */
 				wnd_init_option_ = 1;
+                        else if ((highest_ack_ == -1) &&
+                                (wnd_init_option_ == 1) && (wnd_init_ > 1)
+				&& bugfix_ss_)
+                                /*
+                                 * First packet dropped, so don't use larger
+                                 * initial windows.  Bugfix from Mark Allman.
+                                 */
+                                wnd_init_ = 1;
 			if (highest_ack_ == maxseq_ && restart_bugfix_)
 			       /* 
 				* if there is no outstanding data, don't cut 
